from abc import ABC, abstractmethod
from enum import Enum
from typing import Union


class MemoryDataType(Enum):
    UNKNOWN = 1
    DIALOG = 2
    STRING = 3


class MemorySourceType(Enum):
    UNKNOWN = 1
    LOCAL_FILE = 2
    LOCAL_DB = 3
    REMOTE_FILE = 4
    REMOTE_DB = 5


class BaseMemorySource(ABC):
    """
        BaseMemorySource: memory source base class
    """

    def __init__(self, source_type: MemorySourceType, src: str):
        self.source_type = source_type
        self.src = src

    @abstractmethod
    def read(self) -> bytes:
        pass

    @abstractmethod
    def write(self, data: bytes):
        pass


class BaseMemoryData(ABC):
    """
        BaseMemoryData: memory data base class
    """

    def __init__(self, date_type: MemorySourceType, data: Union[str]):
        self.date_type = date_type
        self.data = data


class BaseMemory(ABC):
    """
        BaseMemory: memory base class
    """

    def __init__(self, data: BaseMemoryData, source: BaseMemorySource):
        self.data = data
        self.source = source

    @abstractmethod
    def recall(self):
        pass

    @abstractmethod
    def update(self):
        pass

    @abstractmethod
    def clear(self):
        pass



from abc import ABC, abstractmethod
from enum import Enum
from typing import Any, List


class RoleType(Enum):
    SYSTEM = "system"
    USER = "user"
    ASSISTANT = "assistant"
    FUNCTION = "function"


class DataCell(object):
    def __init__(self, role: RoleType, content: str, max_tokens: int = None, temperature: float = 0,
                 activated: bool = True, stream: bool = False):
        self.role = role
        self.content = content
        self.max_tokens = max_tokens
        self.temperature = temperature
        self.activated = activated
        self.stream = stream

    def __dict__(self):
        if not self.activated:
            raise Exception("DataCell not activated")
        return {'role': self.role.value, 'content': self.content}

    def activate(self, params: dict[str, Any]):
        if not self.activated:
            self.content = self.content.format(**params)
            self.activated = True


class BasePrompt(ABC):
    """
        BasePrompt: prompt base class
    """

    cell_list: List[DataCell] = None
    ready: bool = False
    temperature: float = 0
    max_tokens: int = None
    stream: bool = False

    def __init__(self, cell_list: List[DataCell]):
        self.cell_list = cell_list

    @abstractmethod
    def fill_params(self, params: dict[str, Any]):
        pass

    @abstractmethod
    def append(self, cell: DataCell):
        pass

    @abstractmethod
    def prompt(self) -> List[dict[str, str]]:
        pass



from oslui.prompts.base import BasePrompt, DataCell, RoleType
from oslui.prompts.chat import ChatPrompt, ChatInput, chat_user_cell, chat_assistant_cell
from oslui.prompts.translate import TranslatePrompt, TranslateInput
from oslui.prompts.classifier import ClassifierPrompt, ClassifierInput
from oslui.prompts.debug import DebugPrompt, DebugInput



from typing import Any

from oslui.agent import BaseAgent
from oslui.llms import BaseLLM
from oslui.prompts import TranslatePrompt


class TranslateAgent(BaseAgent):
    """
        TranslateAgent: translate natural language command to shell command
    """

    def __init__(self, llm: BaseLLM):
        super().__init__(llm=llm, prompt=TranslatePrompt())

    def run(self, params: dict[str, Any] = None) -> str:
        try:
            self.prompt.fill_params(params)
        except Exception as exc:
            add_info = "Parameters complete failed before generating shell commands"
            new_exc = Exception(f"Error occurred: {add_info}")
            new_exc.__cause__ = exc
            raise new_exc

        try:
            response = self.llm.infer(self.prompt)
            result = response["choices"][0]["message"]["content"]
        except Exception as exc:
            add_info = "Something wrong when generating shell commands"
            new_exc = Exception(f"Error occurred: {add_info}")
            new_exc.__cause__ = exc
            raise new_exc

        return result



import os
import platform
import re


def get_environment_variable(variable_name):
    try:
        value = os.environ[variable_name]
        return value
    except KeyError:
        raise Exception(f"environment variable {variable_name} does not exist")


def get_os_type() -> str:
    os_type = "Linux"
    os_name = platform.system()
    if os_name == "Windows":
        os_type = "Windows"
    elif os_name == "Darwin":
        os_type = "Mac"
    elif os_name == "Linux":
        try:
            with open("/etc/os-release", "r") as f:
                for line in f:
                    if line.startswith("PRETTY_NAME"):
                        _, distribution = line.strip().split("=", 1)
                        os_type = distribution.strip('"')
        except FileNotFoundError:
            pass

    return os_type


def get_language_type(text) -> str:
    text = re.sub(r"[^\w\s]", "", text)

    if any("\u4e00" <= c <= "\u9fff" for c in text):
        return "Chinese"
    else:
        return "English"



from typing import Any, List

from pydantic import BaseModel, ValidationError

from oslui.prompts import BasePrompt, DataCell, RoleType

classifier_system_cell = DataCell(
    role=RoleType.SYSTEM,
    content="Please identify which of the following intent the given sentence belongs to."
            "And reply the sequence number(only number) before the corresponding intent."
            "##"
            "If want to perform tasks on computer:"
            "1 Execute a simple shell command"
            "2 Need to write code or execute complex shell program"
            "##"
            "If want to ask questions:"
            "3 Asking a question that only pre-2021 knowledge required"
            "4 Asking a question that needs the latest data",
    activated=False
)

classifier_user_cell = DataCell(
    role=RoleType.USER,
    content="Sentence:{sentence}",
    activated=False
)

classifier_cell_list = [classifier_system_cell, classifier_user_cell]


class ClassifierInput(BaseModel):
    sentence: str


class ClassifierPrompt(BasePrompt):
    params: ClassifierInput = None

    def __init__(self):
        super().__init__(classifier_cell_list)

    def fill_params(self, params: dict[str, Any]):
        try:
            self.params = ClassifierInput(**params)
        except ValidationError as exc:
            add_info = "Parameters verification failed when completing the translate prompt"
            new_exc = Exception(f"Error occurred: {add_info}")
            new_exc.__cause__ = exc
            raise new_exc

        for cell in self.cell_list:
            cell.activate(params)

        self.ready = True

    def append(self, cell: DataCell):
        self.cell_list.append(cell)
        if cell.temperature != 0:
            self.temperature = cell.temperature
        if cell.max_tokens:
            self.max_tokens = cell.max_tokens

    def prompt(self) -> List[dict[str, str]]:
        return [cell.__dict__() for cell in self.cell_list]



from typing import Any

from oslui.agent import BaseAgent
from oslui.llms import BaseLLM
from oslui.prompts import ChatPrompt


class ChatAgent(BaseAgent):
    """
        ChatAgent: answer user's question
    """

    def __init__(self, llm: BaseLLM):
        super().__init__(llm=llm, prompt=ChatPrompt())

    def run(self, params: dict[str, Any] = None):
        try:
            self.prompt.fill_params(params)
        except Exception as exc:
            add_info = "Parameters complete failed before answering chat question"
            new_exc = Exception(f"Error occurred: {add_info}")
            new_exc.__cause__ = exc
            raise new_exc

        try:
            response = self.llm.infer(self.prompt)
        except Exception as exc:
            add_info = "Something wrong when answering chat question"
            new_exc = Exception(f"Error occurred: {add_info}")
            new_exc.__cause__ = exc
            raise new_exc

        return response



from abc import ABC, abstractmethod


class BaseCommand(ABC):
    """
        BaseCommand: command base class
    """

    def __init__(self, content: str):
        self.content = content

    @abstractmethod
    def execute(self):
        pass



import openai

from oslui.llms.base import BaseLLM
from oslui.prompts import BasePrompt
from oslui.utils import get_environment_variable


class OpenAI(BaseLLM):
    def __init__(self, model: str, key: str = None):
        if not key:
            key = get_environment_variable("OPENAI_API_KEY")
        super().__init__(model=model, key=key)

    def infer(self, prompt: BasePrompt) -> str:
        response = openai.ChatCompletion.create(
            model=self.model,
            messages=prompt.prompt(),
            temperature=prompt.temperature,
            max_tokens=prompt.max_tokens,
            stream=prompt.stream,
        )

        return response



from enum import Enum
from typing import Any

from oslui.agent import BaseAgent
from oslui.llms import BaseLLM
from oslui.prompts import ClassifierPrompt


class IntentType(Enum):
    UNKNOWN = 0
    SHELL = 1
    PROGRAM = 2
    IN_QUERY = 3
    OUT_QUERY = 4


class ClassifierAgent(BaseAgent):
    """
        ClassifierAgent: identify user intent
    """

    def __init__(self, llm: BaseLLM):
        super().__init__(llm=llm, prompt=ClassifierPrompt())

    def run(self, params: dict[str, Any] = None):
        try:
            self.prompt.fill_params(params)
        except Exception as exc:
            add_info = "Parameters complete failed before identifying user intent"
            new_exc = Exception(f"Error occurred: {add_info}")
            new_exc.__cause__ = exc
            raise new_exc

        try:
            response = self.llm.infer(self.prompt)
            result = response["choices"][0]["message"]["content"]
        except Exception as exc:
            print(exc)
            add_info = "Something wrong when identifying user intent"
            new_exc = Exception(f"Error occurred: {add_info}")
            new_exc.__cause__ = exc
            raise new_exc

        if result == "1":
            return IntentType.SHELL
        elif result == "2":
            return IntentType.PROGRAM
        elif result == "3":
            return IntentType.IN_QUERY
        elif result == "4":
            return IntentType.OUT_QUERY

        return IntentType.UNKNOWN



import argparse
import logging
import os
import sys

curPath = os.path.abspath(os.path.dirname(__file__))
rootPath = os.path.split(curPath)[0]
sys.path.append(rootPath)

from oslui.tui import ChatTUIType, ChatTUI, DebugTUI
from oslui.llms import OpenAI
from oslui.action import ShellCommand
from oslui.agent import TranslateAgent, ChatAgent, ClassifierAgent, IntentType, DebugAgent
from oslui.utils import get_environment_variable, get_os_type


def main():
    console_handler = logging.StreamHandler()
    console_handler.setLevel(logging.WARNING)
    logging.getLogger('').addHandler(console_handler)

    parser = argparse.ArgumentParser(
        description="Natural Language User Interface for Operating Systems")
    parser.add_argument('lang_cmd', type=str, nargs='*',
                        help="Natural language command")
    parser.add_argument("-i", "--immerse", action="store_true",
                        required=False, help="Immerse mode")
    parser.add_argument("-d", "--debug", action="store_true",
                        required=False, help="Debug for your confusing questions")

    args = parser.parse_args()

    try:
        openai_api_key = get_environment_variable("OPENAI_API_KEY")
    except Exception as e:
        logging.error(str(e))
        exit(1)

    llm = OpenAI(model="gpt-3.5-turbo", key=openai_api_key)

    selected_count = sum([1 for arg in [args.immerse, args.debug] if arg])
    if selected_count > 1:
        parser.error(
            "Parameter conflict: only one of -i or -d can be selected")

    if args.immerse:
        chat_agent = ChatAgent(llm)
        tui = ChatTUI(chat_agent, tui_type=ChatTUIType.CONTINUOUS)
        tui.show()
    elif args.debug:
        debug_agent = DebugAgent(llm)
        tui = DebugTUI(debug_agent)
        tui.show()
    else:
        lang_cmd = ' '.join(args.lang_cmd)
        classifier = ClassifierAgent(llm)
        try:
            params = {"sentence": lang_cmd}
            intent = classifier.run(params)
        except Exception as exc:
            print(exc)
            exit(1)

        if intent == IntentType.SHELL or intent == IntentType.PROGRAM:
            trans_agent = TranslateAgent(llm)
            try:
                os_type = get_os_type()
                params = {"os_type": os_type, "lang_cmd": lang_cmd}
                result = trans_agent.run(params)
            except Exception as exc:
                print(exc)
                exit(1)

            sh_cmd = ShellCommand(result)
            sh_cmd.modify()
            sh_cmd.execute()
        elif intent == IntentType.IN_QUERY or intent == IntentType.OUT_QUERY:
            chat_agent = ChatAgent(llm)
            tui = ChatTUI(chat_agent)
            tui.show(lang_cmd)


if __name__ == "__main__":
    main()



from oslui.action import ShellCommand
from oslui.audit import BaseAuditor


class ShellAuditor(BaseAuditor):
    def __init__(self):
        # 初始化一些审计规则
        super().__init__()
        self.danger_keywords = ['rm', 'chmod', 'chown',
                                '|', '&', '>', '>>', '<', 'eval', '`', '$(', ';']

    def audit(self, command: ShellCommand):
        cmd_content = command.content
        for keyword in self.danger_keywords:
            if keyword in command:
                return False

        # 不允许执行外部命令或绝对路径，只允许执行系统内置的命令
        if '/' in command and not cmd_content.startswith('/bin/') and not cmd_content.startswith('/usr/bin/'):
            return False

        # 不允许包含变量或通配符
        if '$' in command or '*' in command:
            return False

        # 可以添加更多自定义的审计规则，根据您的需求和系统安全要求

        return True



from oslui.agent.base import BaseAgent
from oslui.agent.translate import TranslateAgent
from oslui.agent.chat import ChatAgent
from oslui.agent.debug import DebugAgent
from oslui.agent.classifier import ClassifierAgent, IntentType



from datetime import datetime
from enum import Enum
from typing import Any

from rich.console import Console
from rich.live import Live
from rich.markdown import Markdown
from rich.text import Text

from oslui.agent import ChatAgent
from oslui.prompts import DataCell, chat_assistant_cell, RoleType
from oslui.tui import BaseTUI
from oslui.utils import get_language_type

introduction = """
# Welcome to OSLUI!

"""


class ChatTUIType(Enum):
    SINGLE = 1
    CONTINUOUS = 2


class ChatTUI(BaseTUI):
    input_msg: str = None

    def __init__(self, agent: ChatAgent, tui_type: ChatTUIType = ChatTUIType.SINGLE):
        super().__init__(agent=agent)
        self.tui_type = tui_type

    def show(self, input_msg: str = None):
        if self.tui_type == ChatTUIType.SINGLE:
            self.input_msg = input_msg
            lang_type = get_language_type(self.input_msg)
            params = {"lang_type": lang_type, "question": self.input_msg}
            self.dynamic_refresh(params)
        elif self.tui_type == ChatTUIType.CONTINUOUS:
            console = Console()
            console.clear()
            id_md = Markdown(introduction)
            console.print(id_md, style="bold green")
            try:
                while True:
                    timestamp = datetime.now().strftime("[%H:%M:%S]")
                    timestamp_text = Text(timestamp, style="bright_blue")
                    console.print(timestamp_text + " You :",
                                  style="bold bright_blue")
                    self.input_msg = console.input()

                    if not self.input_msg.strip():
                        continue
                    elif self.input_msg.lower() in ["exit", "quit", "q"]:
                        break
                    else:
                        timestamp = datetime.now().strftime("[%H:%M:%S]")
                        timestamp_text = Text(timestamp, style="green")
                        console.print(timestamp_text + " OSLUI :",
                                      style="bold green")
                        lang_type = get_language_type(self.input_msg)
                        params = {"lang_type": lang_type,
                                  "question": self.input_msg}
                        self.dynamic_refresh(params)
                        self.agent.prompt.append(chat_assistant_cell)
                        self.agent.prompt.append(DataCell(
                            role=RoleType.USER,
                            content="Question:{question}",
                            activated=False
                        ))
            except KeyboardInterrupt:
                pass

    def dynamic_refresh(self, params: dict[str, Any] = None):
        try:
            result, buffer_str = "", ""
            buffer = []
            with Live(vertical_overflow="visible") as live:
                for chunk in self.agent.run(params):
                    if "content" in chunk.choices[0]["delta"]:
                        content = chunk.choices[0]["delta"]["content"]
                        buffer_str += content
                        buffer.append(content)
                        buffer_show = "".join(buffer).strip()
                        if "\n" in content:
                            buffer = []
                            result += buffer_str
                            buffer_str = ""
                            buffer_show = ""

                        live.update(Markdown(result + f'*{buffer_show}*'))
                result += buffer_str
                live.update(Markdown(result))
        except Exception as exc:
            print(exc)
            exit(1)



from typing import Any, List

from pydantic import BaseModel, ValidationError

from oslui.prompts import BasePrompt, DataCell, RoleType

chat_system_cell = DataCell(
    role=RoleType.SYSTEM,
    content="You are a helpful assistant, and you are the best expert to answer user's question."
            "It is recommended to use markdown syntax when necessary."
            "Please use {lang_type} answer questions as detail as possible.",
    activated=False
)

chat_user_cell = DataCell(
    role=RoleType.USER,
    content="Question:{question}",
    activated=False
)

chat_assistant_cell = DataCell(
    role=RoleType.ASSISTANT,
    content="Detailed answer to the previous question"
)

chat_cell_list = [chat_system_cell, chat_user_cell]


class ChatInput(BaseModel):
    question: str
    lang_type: str


class ChatPrompt(BasePrompt):
    params: ChatInput = None

    def __init__(self):
        super().__init__(chat_cell_list)
        self.stream = True

    def fill_params(self, params: dict[str, Any]):
        try:
            self.params = ChatInput(**params)
        except ValidationError as exc:
            add_info = "Parameters verification failed when completing the chat prompt"
            new_exc = Exception(f"Error occurred: {add_info}")
            new_exc.__cause__ = exc
            raise new_exc

        for cell in self.cell_list:
            cell.activate(params)

        self.ready = True

    def append(self, cell: DataCell):
        self.cell_list.append(cell)
        if cell.temperature != 0:
            self.temperature = cell.temperature
        if cell.max_tokens:
            self.max_tokens = cell.max_tokens

    def prompt(self) -> List[dict[str, str]]:
        return [cell.__dict__() for cell in self.cell_list]



from typing import Any, List

from pydantic import BaseModel, ValidationError

from oslui.prompts import BasePrompt, DataCell, RoleType

trans_system_cell = DataCell(
    role=RoleType.SYSTEM,
    content="You are an excellent computer expert, computer OS type is {os_type}."
            "Please generate shell commands that meet user needs"
            "##"
            "Needs:create a new git branch named develop"
            "git branch develop",
    activated=False
)

trans_user_cell = DataCell(
    role=RoleType.USER,
    content="Needs:{lang_cmd}",
    activated=False
)

translate_cell_list = [trans_system_cell, trans_user_cell]


class TranslateInput(BaseModel):
    os_type: str
    lang_cmd: str


class TranslatePrompt(BasePrompt):
    params: TranslateInput = None

    def __init__(self):
        super().__init__(translate_cell_list)

    def fill_params(self, params: dict[str, Any]):
        try:
            self.params = TranslateInput(**params)
        except ValidationError as exc:
            add_info = "Parameters verification failed when completing the translate prompt"
            new_exc = Exception(f"Error occurred: {add_info}")
            new_exc.__cause__ = exc
            raise new_exc

        for cell in self.cell_list:
            cell.activate(params)

        self.ready = True

    def append(self, cell: DataCell):
        self.cell_list.append(cell)
        if cell.temperature != 0:
            self.temperature = cell.temperature
        if cell.max_tokens:
            self.max_tokens = cell.max_tokens

    def prompt(self) -> List[dict[str, str]]:
        return [cell.__dict__() for cell in self.cell_list]



from abc import ABC, abstractmethod
from typing import Any


class BaseAuditor(ABC):
    """
        BaseAuditor: auditor base class
    """

    def __init__(self):
        pass

    @abstractmethod
    def audit(self, command: Any):
        pass



import os
import subprocess

from typing import Any

from oslui.agent import BaseAgent
from oslui.llms import BaseLLM
from oslui.prompts import DebugPrompt
from oslui.utils import get_os_type


class DebugAgent(BaseAgent):
    """
        DebugAgent: debug agent
    """
    command: str = ""
    error_msg: str = ""
    os_type: str = ""
    params: dict[str, Any] = {}

    def __init__(self, llm: BaseLLM):
        super().__init__(llm=llm, prompt=DebugPrompt())

    def get_command(self):
        tty_name = os.ttyname(0)
        tty_id = os.path.basename(tty_name)
        command_log = os.path.expanduser(f"~/.oslui/command_log/{tty_id}.log")
        try:
            with open(command_log, 'r') as file:
                lines = file.readlines()
                if lines:
                    command_line = -1
                    while True:
                        command = lines[command_line].strip()
                        if command == "oslui -d":
                            command_line -= 1
                            continue
                        else:
                            self.command = command
                            break
        except FileNotFoundError:
            pass

    def get_error_msg(self):
        try:
            result = subprocess.run(
                self.command, shell=True, check=True, text=True, capture_output=True)

            # command executed unsuccessfully
            if result.returncode != 0:
                self.error_msg = result.stderr

        except subprocess.CalledProcessError as e:
            self.error_msg = e.stderr

    def get_os_type(self):
        self.os_type = get_os_type()

    def get_params(self):
        self.get_command()
        if self.command == "":
            exc = Exception("Could not get command")
            raise exc
        self.get_error_msg()
        if self.error_msg == "":
            exc = Exception("Could not get error message")
            raise exc
        self.get_os_type()
        if self.os_type == "":
            exc = Exception("Could not get os type")
            raise exc
        self.params["command"] = self.command
        self.params["error_msg"] = self.error_msg.replace(
            '"', "'").replace('\n', ' ')
        self.params["os_type"] = self.os_type

    def run(self, params: dict[str, Any] = None):
        self.get_params()
        try:
            self.prompt.fill_params(self.params)
        except Exception as exc:
            add_info = "Parameters complete failed before debugging"
            new_exc = Exception(f"Error occurred: {add_info}")
            new_exc.__cause__ = exc
            raise new_exc

        try:
            response = self.llm.infer(self.prompt)
        except Exception as exc:
            add_info = "Something wrong when debugging"
            new_exc = Exception(f"Error occurred: {add_info}")
            new_exc.__cause__ = exc
            raise new_exc

        return response



import os
from setuptools import setup, find_packages
from setuptools.command.install import install
from shutil import copyfile

class InstallScriptCommand(install):
    def run(self):
        install.run(self)
        self.install_script()

    def install_script(self):
        home_dir = os.path.expanduser("~")
        target_dir = os.path.join(home_dir, ".oslui")
        if not os.path.exists(target_dir):
            os.makedirs(target_dir)

        script_path = os.path.join(target_dir, "record_command.sh")
        copyfile("oslui/scripts/record_command.sh", script_path)

setup(
    name='oslui',
    version='0.1.1',
    url='https://github.com/BalianWang/OSLUI',
    author='BalianWang',
    author_email='balian.wang1997@gmail.com',
    description='Natural Language User Interface for Operating Systems',
    packages=find_packages(),
    install_requires=[
        'openai',
        'google-search-results',
    ],
    entry_points={
        'console_scripts': [
            'oslui = oslui.main:main',
        ],
    },
    cmdclass={
        'install': InstallScriptCommand,
    },
    license='MIT',
    python_requires='>=3'
)



from abc import ABC, abstractmethod
from typing import Any

from oslui.llms import BaseLLM
from oslui.prompts import BasePrompt


class BaseAgent(ABC):
    """
        BaseAgent: OS Agent base class
    """

    def __init__(self, llm: BaseLLM, prompt: BasePrompt):
        self.llm = llm
        self.prompt = prompt

    @abstractmethod
    def run(self, params: dict[str, Any] = None):
        pass



from typing import Any, List

from pydantic import BaseModel, ValidationError

from oslui.prompts import BasePrompt, DataCell, RoleType

debug_system_cell = DataCell(
    role=RoleType.SYSTEM,
    content="You are a helpful assistant, and you are are the best computer expert in the world."
            "An error occurred when the user executed shell command on the computer."
            "Please give the best solution or suggestion based on the error message."
            "It is recommended to use markdown syntax when necessary.",
    activated=False
)

debug_user_cell = DataCell(
    role=RoleType.USER,
    content="Shell command: {command},"
            "Error message: {error_msg},"
            "OS: {os_type}",
    activated=False
)

debug_cell_list = [debug_system_cell, debug_user_cell]


class DebugInput(BaseModel):
    command: str
    error_msg: str
    os_type: str


class DebugPrompt(BasePrompt):
    params: DebugInput = None

    def __init__(self):
        super().__init__(debug_cell_list)
        self.stream = True

    def fill_params(self, params: dict[str, Any]):
        try:
            self.params = DebugInput(**params)
        except ValidationError as exc:
            add_info = "Parameters verification failed when completing the debug prompt"
            new_exc = Exception(f"Error occurred: {add_info}")
            new_exc.__cause__ = exc
            raise new_exc

        for cell in self.cell_list:
            cell.activate(params)

        self.ready = True

    def append(self, cell: DataCell):
        self.cell_list.append(cell)
        if cell.temperature != 0:
            self.temperature = cell.temperature
        if cell.max_tokens:
            self.max_tokens = cell.max_tokens

    def prompt(self) -> List[dict[str, str]]:
        return [cell.__dict__() for cell in self.cell_list]



from rich.live import Live
from rich.markdown import Markdown

from oslui.agent import DebugAgent
from oslui.tui import BaseTUI


class DebugTUI(BaseTUI):
    def __init__(self, agent: DebugAgent):
        super().__init__(agent=agent)

    def show(self, input_msg: str = None):
        self.dynamic_refresh()

    def dynamic_refresh(self):
        try:
            result, buffer_str = "", ""
            buffer = []
            with Live(vertical_overflow="visible") as live:
                for chunk in self.agent.run():
                    if "content" in chunk.choices[0]["delta"]:
                        content = chunk.choices[0]["delta"]["content"]
                        buffer_str += content
                        buffer.append(content)
                        buffer_show = "".join(buffer).strip()
                        if "\n" in content:
                            buffer = []
                            result += buffer_str
                            buffer_str = ""
                            buffer_show = ""

                        live.update(Markdown(result + f'*{buffer_show}*'))
                result += buffer_str
                live.update(Markdown(result))
        except Exception as exc:
            print(exc)
            exit(1)



from abc import ABC, abstractmethod

from oslui.agent import BaseAgent


class BaseTUI(ABC):
    """
        BaseTUI: terminal user interface base class
    """

    def __init__(self, agent: BaseAgent):
        self.agent = agent

    @abstractmethod
    def show(self, input_msg: str = None):
        pass



from abc import ABC, abstractmethod

from oslui.prompts import BasePrompt


class BaseLLM(ABC):
    """
        BaseLLM: large language model base class
    """

    def __init__(self, model: str, key: str = None):
        self.model = model
        self.key = key

    @abstractmethod
    def infer(self, prompt: BasePrompt):
        pass
